/**************************************************************

frame.c (C-Munipack project)
Table of apertures (phootmetry file)
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "console.h"
#include "cmpack_common.h"
#include "cmpack_allfile.h"
#include "allframes.h"

/*************************   Constants  ***********************/

#define ALLOC_BY 	64

static void frame_init(CmpackAllFrame *frame, double jd)
{
	memset(frame, 0, sizeof(CmpackAllFrame));
	frame->jd = jd;
}

static void frame_clear(CmpackAllFrame *frame)
{
	cmpack_free(frame->list);
	frame->list = NULL;
	frame->count = frame->capacity = 0;
}

static void frame_copy(CmpackAllFrame *dst, const CmpackAllFrame *src)
{
	dst->capacity = dst->count = src->count;
	dst->list = (CmpackAllStar*)cmpack_realloc(dst->list, dst->capacity*sizeof(CmpackAllStar));
	memcpy(dst->list, src->list, src->count*sizeof(CmpackAllStar));
}

/********************   Public functions    ***********************/

/* Init table of apertures */
void all_tab_init(CmpackAllFrameTab *tab)
{
	memset(tab, 0, sizeof(CmpackAllFrameTab));
}	

/* Free allocated memory */
void all_tab_clear(CmpackAllFrameTab *tab)
{
	int j;
	
	for (j=0; j<tab->count; j++) 
		frame_clear(&tab->list[j]);
	cmpack_free(tab->list);
	tab->list = NULL;
	tab->count = tab->capacity = tab->stars = 0;
}	

/* Make copy of header */
void all_tab_copy(CmpackAllFrameTab *dst, const CmpackAllFrameTab *src)
{
	int i;
	
	all_tab_clear(dst);
	if (src->count>0) {
		dst->capacity = src->count;
		dst->list = (CmpackAllFrame*)cmpack_malloc(dst->capacity*sizeof(CmpackAllFrame));
		dst->count = 0;
		dst->stars = 0;
		for (i=0; i<src->count; i++) {
			frame_copy(&dst->list[i], &src->list[i]);
			if (dst->list[i].count > dst->stars)
				dst->stars = dst->list[i].count;
		}
		dst->count = src->count;
	}
}

/* Write table */
void all_tab_write(CmpackAllFrameTab *tab, FILE *to)
{
	int i, j;
	
	for (i=0; i<tab->count; i++) {
		CmpackAllFrame *frame = &tab->list[i];
		fprintf(to, "%.5f", frame->jd); 
		for (j=0; j<frame->count; j++) {
			CmpackAllStar *star = &frame->list[j];
			fprintf(to, " %.4f %.4f", 
				((star->mask & CMPACK_MASK_MAG) ? star->mag : 99.999),
				((star->mask & CMPACK_MASK_ERR) ? star->err : 9.999));
		}
		fprintf(to, "\n"); 
	}
}

/* Add new section */
int all_tab_add_frame(CmpackAllFrameTab *tab, double jd)
{
	if (tab->count>=tab->capacity) {
		tab->capacity += ALLOC_BY;
		tab->list = (CmpackAllFrame*)cmpack_realloc(tab->list, tab->capacity*sizeof(CmpackAllFrame));
	}
	frame_init(&tab->list[tab->count], jd);
	return tab->count++;
}

/* Add new section */
void all_tab_set_star(CmpackAllFrameTab *tab, int frame, int star, int mask, double mag, double err)
{
	CmpackAllFrame *f;

	if (frame>=0 && frame<tab->count) {
		f = &tab->list[frame];
		if (star>=0 && star<65536) {
			if (star>=f->capacity) {
				f->capacity = ALLOC_BY*((star/ALLOC_BY)+1);
				f->list = (CmpackAllStar*)cmpack_realloc(f->list, f->capacity*sizeof(CmpackAllStar));
			}
			if (star > f->count)
				memset(f->list + f->count, 0, (star-f->count)*sizeof(CmpackAllStar));
			f->list[star].mask = mask;
			f->list[star].mag = mag;
			f->list[star].err = err;
			if (star >= f->count)
				f->count = star+1;
		}
		if (tab->stars <= star)
			tab->stars = star;
	}
}

/* Add new section */
int all_tab_get_star(CmpackAllFrameTab *tab, int frame, int star, int *mask, double *mag, double *err)
{
	CmpackAllFrame *f;

	if (frame>=0 && frame<tab->count) {
		f = &tab->list[frame];
		if (star>=0 && star<f->count) {
			if (mask) *mask = f->list[star].mask;
			if (mag) *mag = f->list[star].mag;
			if (err) *err = f->list[star].err;
			return 1;
		}
	}
	if (mask) *mask = 0;
	return 0;
}
