/**************************************************************

allstars.h (C-Munipack project)
Table of stars
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef ALLFILE_FRAMES_H
#define ALLFILE_FRAMES_H

#include "comfun.h"

/*********************   Datatypes   *******************/

/* Star data (single measurement) */
struct _CmpackAllStar
{
	int mask;
	double mag, err;
};
typedef struct _CmpackAllStar CmpackAllStar;

/* Frame definition */
struct _CmpackAllFrame
{
	double jd;						/**< Julian date of observation */
	int count;						/**< Number of frames */
	int capacity;					/**< Number of allocated records */
	CmpackAllStar *list;			/**< Table of stars */
};
typedef struct _CmpackAllFrame CmpackAllFrame;

/* Table of frames */
struct _CmpackAllFrameTab
{
	int stars;						/**< Number of stars */
	int count;						/**< Number of frames */
	int capacity;					/**< Number of allocated records */
	CmpackAllFrame *list;			/**< List of frames */
};
typedef struct _CmpackAllFrameTab CmpackAllFrameTab;

/********************   Public functions    ***********************/

/* Init table */
void all_tab_init(CmpackAllFrameTab *tab);
	
/* Clear table */
void all_tab_clear(CmpackAllFrameTab *tab);

/* Copy table */
void all_tab_copy(CmpackAllFrameTab *dst, const CmpackAllFrameTab *src);

/* Dump table */
void all_tab_dump(void *ctx, CmpackAllFrameTab *dst);

/* Write table */
void all_tab_write(CmpackAllFrameTab *dst, FILE *to);

/* Add new frame */
int all_tab_add_frame(CmpackAllFrameTab *tab, double jd);

/* Add new star */
void all_tab_set_star(CmpackAllFrameTab *tab, int frame, int star, int mask, double mag, double err);

/* Add new star */
int all_tab_get_star(CmpackAllFrameTab *tab, int frame, int star, int *mask, double *mag, double *err);

#endif
