/**************************************************************

phot_stars.h (C-Munipack project)
Table of stars
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef CAT_SELECT_H
#define CAT_SELECT_H

#include "context.h"
#include "cmpack_console.h"

/*********************   Datatypes   *******************/

/* Selection table */
typedef struct _CmpackCatSelectionTable CmpackCatSelectionTable;

/********************   Public functions    ***********************/

/* Init the table */
CmpackCatSelectionTable *cat_seltab_create(void);

/* Free allocated memory */
void cat_seltab_free(CmpackCatSelectionTable *tab);

/* Clear the table */
void cat_seltab_clear(CmpackCatSelectionTable *tab);
	
/* Dump the table */
void cat_seltab_dump(CmpackCatSelectionTable *tab, CmpackConsole *ctx);
	
/* Write all stars to file */
void cat_seltab_write(CmpackCatSelectionTable *tab, FILE *to);

/* Copy table of stars */
void cat_seltab_copy(CmpackCatSelectionTable *dsttab, const CmpackCatSelectionTable *srcttab);
	
/* Get pointer to selection by type */
CmpackCatSelection *cat_seltab_get(CmpackCatSelectionTable *tab, CmpackSelectionType type);

/* Set selection type */
void cat_seltab_set_star(CmpackCatSelectionTable *tab, int star_id, CmpackSelectionType type);

/* Find star by its identifier, returns the selection type */
CmpackSelectionType cat_seltab_get_star(CmpackCatSelectionTable *tab, int star_id);

/* Creates a new star and returns its index */
void cat_select_append(CmpackCatSelection *sel, const int *list, int count);

/* Creates a new star and returns its index */
int cat_select_get(CmpackCatSelection *sel, int *buf, int bufsize);

/* Creates a new star and returns its index */
void cat_select_clear(CmpackCatSelection *sel);

/* Deletes a record from a selection */
void cat_select_delete(CmpackCatSelection *sel, int index);

#endif
