/**************************************************************

comfun.c (C-Munipack project)
Common functions, macros, etc.
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include <float.h>

#ifndef _WIN32
#include <sys/stat.h>
#include <sys/types.h>
#endif

#include "comfun.h"
#include "cmpack_common.h"

#define MAXBUFFER	65536

void *image_new(int nx, int ny, CmpackBitpix bitpix)
{
	switch (bitpix)
	{
	case CMPACK_BITPIX_SCHAR:  
	case CMPACK_BITPIX_UCHAR:  
		return cmpack_malloc(nx*ny*sizeof(char));
	case CMPACK_BITPIX_SSHORT:
	case CMPACK_BITPIX_USHORT:
		return cmpack_malloc(nx*ny*sizeof(short));
	case CMPACK_BITPIX_SLONG:
	case CMPACK_BITPIX_ULONG:
		return cmpack_malloc(nx*ny*sizeof(long));
	case CMPACK_BITPIX_FLOAT:
		return cmpack_malloc(nx*ny*sizeof(float));
	case CMPACK_BITPIX_DOUBLE:
		return cmpack_malloc(nx*ny*sizeof(double));
	default:
		return NULL;
	}
}

int pixrange(int bitpix, double *minvalue, double *maxvalue)
{
	switch (bitpix) 
	{
	case CMPACK_BITPIX_SCHAR:  
		if (minvalue) *minvalue = CHAR_MIN;
		if (maxvalue) *maxvalue = CHAR_MAX;
		return 1;
	case CMPACK_BITPIX_UCHAR:  
		if (minvalue) *minvalue = 0.0;
		if (maxvalue) *maxvalue = UCHAR_MAX;
		return 1;
	case CMPACK_BITPIX_SSHORT:
		if (minvalue) *minvalue = SHRT_MIN;
		if (maxvalue) *maxvalue = SHRT_MAX;
		return 1;
	case CMPACK_BITPIX_USHORT:
		if (minvalue) *minvalue = 0.0;
		if (maxvalue) *maxvalue = USHRT_MAX;
		return 1;
	case CMPACK_BITPIX_SLONG:
		if (minvalue) *minvalue = LONG_MIN;
		if (maxvalue) *maxvalue = LONG_MAX;
		return 1;
	case CMPACK_BITPIX_ULONG:
		if (minvalue) *minvalue = 0.0;
		if (maxvalue) *maxvalue = ULONG_MAX;
		return 1;
	case CMPACK_BITPIX_FLOAT:
		if (minvalue) *minvalue = FLT_MIN;
		if (maxvalue) *maxvalue = FLT_MAX;
		return 1;
	case CMPACK_BITPIX_DOUBLE:
		if (minvalue) *minvalue = DBL_MIN;
		if (maxvalue) *maxvalue = DBL_MAX;
		return 1;
	default:
		if (minvalue) *minvalue = 0.0;
		if (maxvalue) *maxvalue = 0.0;
		return 0;
	}
}

const char *pixformat(int bitpix)
{
	switch (bitpix) 
	{
	case CMPACK_BITPIX_UCHAR:  
		return "Unsigned char (1 byte)";
	case CMPACK_BITPIX_SCHAR:  
		return "Signed char (1 byte)";
	case CMPACK_BITPIX_SSHORT:
		return "Signed short (2 bytes)";
	case CMPACK_BITPIX_USHORT:
		return "Unsigned short (2 bytes)";
	case CMPACK_BITPIX_SLONG:
		return "Signed long (4 bytes)";
	case CMPACK_BITPIX_ULONG:
		return "Unsigned long (4 bytes)";
	case CMPACK_BITPIX_FLOAT:
		return "Single precision FP (4 bytes)";
	case CMPACK_BITPIX_DOUBLE:
		return "Double precision FP (8 bytes)";
	default:
		return "Unknown data format";
	}
}

int strtosign(const char *str, char **endptr)
{
	const char *ptr;

	if (!str) {
		if (endptr)
			*endptr = NULL;
		return 0;
	}
	for (ptr=str; *ptr==' '; ptr++) {
		if (*ptr=='-') {
			if (endptr)
				*endptr = (char*)ptr+1;
			return -1;
		} 
		if (*ptr=='+') {
			if (endptr)
				*endptr = (char*)ptr+1;
			return +1;
		}
		break;
	}
	if (endptr)
		*endptr = (char*)ptr;
	return 1;
}

long cmpack_get_param_long(CmpackParam param)
{
	switch (param)
	{
	case CMPACK_PARAM_JD_PRECISION:
		return JD_PRECISION;
	case CMPACK_PARAM_POS_PRECISION:
		return POS_PRECISION;
	case CMPACK_PARAM_EXP_PRECISION:
		return EXP_PRECISION;
	case CMPACK_PARAM_TEMP_PRECISION:
		return TEMP_PRECISION;
	case CMPACK_PARAM_AMASS_PRECISION:
		return AMASS_PRECISION;
	case CMPACK_PARAM_ALT_PRECISION:
		return ALT_PRECISION;
	default:
		return 0;
	}
}
