/**************************************************************

context.c (C-Munipack project)
Context related functions
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "cmpack_common.h"
#include "context.h"

/* Create and initialize the context */
void *gc_create(int size)
{
	CmpackContext *ctx = cmpack_calloc(1, size);
	if (ctx) {
		ctx->refcnt = 1;
		ctx->clean = gc_clean;
	}
	return ctx;
}

/* There are no allocated buffers */
void gc_clean(void *ctx)
{
}

/* Increment the reference counter */
void *cmpack_addref(void *ctx)
{
	if (ctx) {
		assert(((CmpackContext*)ctx)->refcnt > 0);
		((CmpackContext*)ctx)->refcnt++;
	}
	return ctx;
}

/* Decrement the reference counter and when it reaches zero, 
 * free the allocated memory
 */
void cmpack_unref(void *ctx)
{
	if (ctx) {
		assert(((CmpackContext*)ctx)->refcnt > 0);
		((CmpackContext*)ctx)->refcnt--;
		if (((CmpackContext*)ctx)->refcnt==0) {
			((CmpackContext*)ctx)->clean(ctx);
			cmpack_free(ctx);
		}
	}
}
