/**************************************************************

tcor.c (C-Munipack project)
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "comfun.h"
#include "trajd.h"
#include "console.h"
#include "ccdfile.h"
#include "cmpack_common.h"
#include "cmpack_tcorr.h"

/******************   Private data types   **************************/

/* Time correction context */
struct _CmpackTimeCorr
{
	CmpackContext gc;			/**< Global context */
	CmpackConsole *con;			/**< Console */
	double tcor;    			/**< Time correction in seconds */
};

/*******************   LOCAL FUNCTIONS   *******************************/

/* Free allocated memory */
static void tcorr_clean(void *ctx)
{
	CmpackTimeCorr *lc = (CmpackTimeCorr*)ctx;

	cmpack_unref(lc->con);
	gc_clean(&lc->gc);
}

/*********************   PUBLIC FUNCTIONS   ************************/

/* Context initialization */
CmpackTimeCorr *cmpack_tcorr_init(void)
{
	CmpackTimeCorr *lc = gc_create(sizeof(CmpackTimeCorr));
	lc->gc.clean = &tcorr_clean;
	return lc;
}

/* Attach console */
void cmpack_tcorr_set_console(CmpackTimeCorr *ctx, CmpackConsole *con)
{
	if (con!=ctx->con) {
		cmpack_unref(ctx->con);
		ctx->con = cmpack_addref(con);
	}
}

/* Set output image data format */
void cmpack_tcorr_set_corr(CmpackTimeCorr *lc, double corr)
{
	lc->tcor = corr;
}

/* Set output image data format */
double cmpack_tcorr_get_corr(CmpackTimeCorr *lc)
{
	return lc->tcor;
}

/* Time correction */
int cmpack_tcorr(CmpackTimeCorr *lc, CmpackCcdFile *file)
{
	CmpackDateTime dt;
	double jd;

	/* Check source file */
	if (!file) {
		printout(lc->con, 0, "Invalid frame context.");
		return CMPACK_ERR_INVALID_PAR;
	}

	/* Read date and time */
	if (cmpack_ccd_get_datetime(file, &dt)!=0)
		return CMPACK_ERR_READ_ERROR;
	jd = cmpack_encodejd(&dt);
	if (jd<=0.0) 
		return CMPACK_ERR_INVALID_DATE;

	/* Time correction */
	printpard(lc->con, "Old JD", 1, jd, 6);
	if (jd>0)
		jd += lc->tcor/86400.0;
	printpard(lc->con, "New JD", 1, jd, 6);

	/* Write date and time */
	cmpack_decodejd(jd, &dt);
	if (ccd_set_datetime(file, &dt)!=0)
		return CMPACK_ERR_WRITE_ERROR;

	/* Update file header */
	ccd_write_history(file, "Time corrected.");
	return 0;
}

/* Time correction */
int cmpack_tcorr_ex(CmpackTimeCorr *lc, CmpackCcdFile *src, CmpackCcdFile *dst)
{
	int res;
	
	res = cmpack_ccd_copy(src, dst);
	if (res==0)
		res = cmpack_tcorr(lc, dst);
	return res;
}
