/**************************************************************

addfiles_dlg.cpp (C-Munipack project)
The 'Add files' dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>

#include "addfiles_dlg.h"
#include "addfilter_dlg.h"
#include "project.h"
#include "configuration.h"
#include "utils.h"
#include "main.h"

CAddFilesDlg::CAddFilesDlg(GtkWindow *pParent)
{
	// Dialog with buttons
	m_pDlg = gtk_file_chooser_dialog_new("Add files", pParent, GTK_FILE_CHOOSER_ACTION_OPEN,
		GTK_STOCK_CLOSE, GTK_RESPONSE_REJECT, GTK_STOCK_ADD, GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(m_pDlg), true);

	// Dialog icon
	gchar *icon = get_icon_file("addfiles");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Preview widget
	gtk_file_chooser_set_preview_widget(GTK_FILE_CHOOSER(m_pDlg), 
		GTK_WIDGET(g_object_ref(m_Preview.Handle())));
	m_Preview.SetTypeMask(FILE_TYPE_IMAGE);
	m_Preview.SetParent(GTK_WINDOW(m_pDlg));

	// Signals
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);
	g_signal_connect(G_OBJECT(m_pDlg), "update-preview", G_CALLBACK(update_preview), this);
}

CAddFilesDlg::~CAddFilesDlg()
{
	gtk_widget_destroy(m_pDlg);
}

void CAddFilesDlg::Execute()
{
	char *path;

	// Go to last folder
	path = CConfig::GetStr("AddFiles", "Folder", NULL);
	if (path) {
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(m_pDlg), path);
		g_free(path);
	}

	gtk_file_chooser_set_preview_widget_active(GTK_FILE_CHOOSER(m_pDlg), 
		!CConfig::GetBool("Environment", "NoPreviews"));

	gtk_dialog_run(GTK_DIALOG(m_pDlg));

	Global.pProject->Save();
}

void CAddFilesDlg::update_preview(GtkFileChooser *pChooser, CAddFilesDlg *pMe)
{
	pMe->OnUpdatePreview(pChooser);
}

void CAddFilesDlg::OnUpdatePreview(GtkFileChooser *pChooser)
{
	if (!CConfig::GetBool("Environment", "NoPreviews")) {
		gchar *file = gtk_file_chooser_get_preview_filename(pChooser);
		if (file) {
			m_Preview.Update(file);
			g_free(file);
		} else {
			m_Preview.Clear();
		}
	}
}

void CAddFilesDlg::response_dialog(GtkDialog *pDlg, gint response_id, CAddFilesDlg *pMe)
{
	if (!pMe->OnCloseQuery(response_id)) 
		g_signal_stop_emission_by_name(pDlg, "response");
}

bool CAddFilesDlg::OnCloseQuery(gint response_id)
{
	gchar *path;
	GSList *filelist;
	CAddFilterDlg *pFilterDlg;

	if (response_id == GTK_RESPONSE_ACCEPT) {
		path = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(m_pDlg));
		CConfig::SetStr("AddFiles", "Folder", (path ? path : ""));
		filelist = gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(m_pDlg));
		pFilterDlg = new CAddFilterDlg(GTK_WINDOW(m_pDlg));
		pFilterDlg->Execute(filelist);
		delete pFilterDlg;
		g_slist_free(filelist);
		g_free(path);
		return false;
	}
	return true;
}

