/**************************************************************

airmass_dlg.cpp (C-Munipack project)
Air-mass coefficient dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <cmunipack.h>

#include "utils.h"
#include "configuration.h"
#include "airmass_dlg.h"
#include "main_dlg.h"

//-------------------------   JD CONVERTER DIALOG   --------------------------------

bool	CAirMassDlg::ms_NoInput = true;
bool	CAirMassDlg::ms_ValidJD = false;
bool	CAirMassDlg::ms_ValidRA = false;
bool	CAirMassDlg::ms_ValidDec = false;
bool	CAirMassDlg::ms_ValidLon = false;
bool	CAirMassDlg::ms_ValidLat = false;
double	CAirMassDlg::ms_LastRA = 0;
double	CAirMassDlg::ms_LastDec = 0;
double	CAirMassDlg::ms_LastJD = 0;
double	CAirMassDlg::ms_LastLon = 0;
double	CAirMassDlg::ms_LastLat = 0;

CAirMassDlg::CAirMassDlg(GtkWindow *pParent)
{
	char buf[256];
	GtkWidget *tbox, *label;

	// Dialog with buttons
	m_pDlg = gtk_dialog_new_with_buttons("Air-mass coefficient", pParent, 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
		GTK_STOCK_REFRESH, GTK_RESPONSE_NONE, GTK_STOCK_CLOSE, 
		GTK_RESPONSE_CLOSE, NULL);

	// Dialog icon
	gchar *icon = get_icon_file("airmass");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Object's coordinates
	tbox = gtk_table_new(6, 3, FALSE);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(m_pDlg)->vbox), tbox);
	gtk_container_set_border_width(GTK_CONTAINER(tbox), 8);
	gtk_table_set_row_spacings(GTK_TABLE(tbox), 4);
	gtk_table_set_row_spacing(GTK_TABLE(tbox), 1, 16);
	gtk_table_set_row_spacing(GTK_TABLE(tbox), 3, 16);
	gtk_table_set_row_spacing(GTK_TABLE(tbox), 4, 16);
	gtk_table_set_col_spacings(GTK_TABLE(tbox), 8);

	label = gtk_label_new("Object - right ascension");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 0, 1, 0, 1);
	m_RA = gtk_entry_new_with_max_length(32);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_RA, 1, 2, 0, 1);
	label = gtk_label_new("[hh mm ss]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 2, 3, 0, 1);
	if (ms_ValidRA && cmpack_ratostr(ms_LastRA, buf, 256)==0) 
		gtk_entry_set_text(GTK_ENTRY(m_RA), buf);
	else
		gtk_entry_set_text(GTK_ENTRY(m_RA), "");

	label = gtk_label_new("- declination");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 0, 1, 1, 2);
	m_Dec = gtk_entry_new_with_max_length(32);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Dec, 1, 2, 1, 2);
	label = gtk_label_new("[\xC2\xB1""ddd mm ss]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 2, 3, 1, 2);
	if (ms_ValidDec && cmpack_dectostr(ms_LastDec, buf, 256)==0) 
		gtk_entry_set_text(GTK_ENTRY(m_Dec), buf);
	else
		gtk_entry_set_text(GTK_ENTRY(m_Dec), "");

	// Observer's coordinates
	label = gtk_label_new("Observer - longitude");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 0, 1, 2, 3);
	m_Lon = gtk_entry_new_with_max_length(32);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Lon, 1, 2, 2, 3);
	label = gtk_label_new("[\xC2\xB1""ddd mm ss]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 2, 3, 2, 3);
	if (ms_ValidLon && cmpack_lontostr(ms_LastLon, buf, 256)==0) 
		gtk_entry_set_text(GTK_ENTRY(m_Lon), buf);
	else
		gtk_entry_set_text(GTK_ENTRY(m_Lon), "");

	label = gtk_label_new("- latitude");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 0, 1, 3, 4);
	m_Lat = gtk_entry_new_with_max_length(32);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Lat, 1, 2, 3, 4);
	label = gtk_label_new("[\xC2\xB1""ddd mm ss]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 2, 3, 3, 4);
	if (ms_ValidLat && cmpack_lattostr(ms_LastLat, buf, 256)==0) 
		gtk_entry_set_text(GTK_ENTRY(m_Lat), buf);
	else
		gtk_entry_set_text(GTK_ENTRY(m_Lat), "");

	label = gtk_label_new("Julian date");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 0, 1, 4, 5);
	m_JD = gtk_entry_new_with_max_length(32);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_JD, 1, 2, 4, 5);
	label = gtk_label_new("[24xxxxx.xxx]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 2, 3, 4, 5);
	if (ms_ValidJD) {
		sprintf(buf, "%.7f", ms_LastJD);
		gtk_entry_set_text(GTK_ENTRY(m_JD), buf);
	}

	label = gtk_label_new("Air-mass coefficient");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 0, 1, 5, 6);
	m_AMass = gtk_entry_new_with_max_length(32);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_AMass, 1, 2, 5, 6);
	gtk_editable_set_editable(GTK_EDITABLE(m_AMass), false);

	SetData();

	gtk_widget_show_all(tbox);

	// Signals:
	GtkWidget *btn = get_dialog_widget_by_response(m_pDlg, GTK_RESPONSE_NONE);
	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(button_clicked), this);
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);
}

CAirMassDlg::~CAirMassDlg()
{
	gtk_widget_destroy(m_pDlg);
}

void CAirMassDlg::response_dialog(GtkDialog *pDlg, gint response_id, CAirMassDlg *pMe)
{
	if (response_id == GTK_RESPONSE_NONE) 
		g_signal_stop_emission_by_name(pDlg, "response");
}

void CAirMassDlg::button_clicked(GtkWidget *pButton, CAirMassDlg *pMe)
{
	int response_id;
	
	response_id = gtk_dialog_get_response_for_widget(GTK_DIALOG(pMe->m_pDlg), pButton);
	if (response_id == GTK_RESPONSE_NONE) {
		pMe->GetData();
		pMe->SetData();
	}
}

void CAirMassDlg::Execute()
{
	gtk_dialog_run(GTK_DIALOG(m_pDlg));
	CConfig::Flush();
}

void CAirMassDlg::GetData()
{
	double ra, dec, lon, lat;
	const char *text;
	char *buf, *ptr;

	ms_NoInput = true;

	// Read object's coordinates
	text = gtk_entry_get_text(GTK_ENTRY(m_RA));
	ms_NoInput = ms_NoInput && (strlen(text)==0);
	if (cmpack_strtora(text, &ra)==0) {
		ms_ValidRA = true;
		ms_LastRA = ra;
	} else
		ms_ValidRA = false;
	text = gtk_entry_get_text(GTK_ENTRY(m_Dec));
	ms_NoInput = ms_NoInput && (strlen(text)==0);
	if (cmpack_strtodec(text, &dec)==0) {
		ms_ValidDec = true;
		ms_LastDec = dec;
	} else
		ms_ValidDec = false;
	// Read observer's coordinates
	text = gtk_entry_get_text(GTK_ENTRY(m_Lon));
	ms_NoInput = ms_NoInput && (strlen(text)==0);
	if (cmpack_strtolon(text, &lon)==0) {
		ms_ValidLon = true;
		ms_LastLon = lon;
	} else
		ms_ValidLon = false;
	text = gtk_entry_get_text(GTK_ENTRY(m_Lat));
	ms_NoInput = ms_NoInput && (strlen(text)==0);
	if (cmpack_strtolat(text, &lat)==0) {
		ms_ValidLat = true;
		ms_LastLat = lat;
	} else
		ms_ValidLat = false;
	// Read julian date
	text = gtk_entry_get_text(GTK_ENTRY(m_JD));
	ms_NoInput = ms_NoInput && (strlen(text)==0);
	if (strlen(text)>0) {
		buf = g_strdup(text);
		ptr = strchr(buf, ',');
		if (ptr)
			*ptr = '.';
		ms_LastJD = atof(buf);
		ms_ValidJD = true;
	} else 
		ms_ValidJD = false;
}

void CAirMassDlg::SetData()
{
	double airmass;
	char buf[256];

	if (ms_NoInput) {
		gtk_entry_set_text(GTK_ENTRY(m_AMass), "");
		return;
	}
	if (!ms_ValidRA || cmpack_ratostr(ms_LastRA, buf, 256)!=0) {
		gtk_entry_set_text(GTK_ENTRY(m_AMass), "Invalid right ascension");
		return;
	} 
	gtk_entry_set_text(GTK_ENTRY(m_RA), buf);

	if (!ms_ValidDec || cmpack_ratostr(ms_LastDec, buf, 256)!=0) {
		gtk_entry_set_text(GTK_ENTRY(m_AMass), "Invalid declination");
		return;
	}
	gtk_entry_set_text(GTK_ENTRY(m_Dec), buf);

	if (!ms_ValidLon || cmpack_lontostr(ms_LastLon, buf, 256)!=0) {
		gtk_entry_set_text(GTK_ENTRY(m_AMass), "Invalid longitude");
		return;
	}
	gtk_entry_set_text(GTK_ENTRY(m_Lon), buf);

	if (!ms_ValidLat || cmpack_lattostr(ms_LastLat, buf, 256)!=0) {
		gtk_entry_set_text(GTK_ENTRY(m_AMass), "Invalid latitude");
		return;
	}
	gtk_entry_set_text(GTK_ENTRY(m_Lat), buf);

	if (!ms_ValidJD) {
		gtk_entry_set_text(GTK_ENTRY(m_AMass), "Invalid julian date");
		return;
	}
	sprintf(buf, "%.7f", ms_LastJD);
	gtk_entry_set_text(GTK_ENTRY(m_JD), buf);

	if (ms_ValidRA && ms_ValidDec && ms_ValidJD && ms_ValidLon && ms_ValidLat) {
		airmass = cmpack_airmass(ms_LastJD, ms_LastRA, ms_LastDec, ms_LastLon, ms_LastLat, NULL);
		if (airmass<0.0) 
			strcpy(buf, "Object is below horizon");
		else
			sprintf(buf, "%.3f", airmass);
		gtk_entry_set_text(GTK_ENTRY(m_AMass), buf);
	}
}
