/**************************************************************

chart_dlg.cpp (C-Munipack project)
The 'Plot chart' dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>

#include "project.h"
#include "chart.h"
#include "chart_dlg.h"
#include "main.h"
#include "utils.h"
#include "configuration.h"
#include "chartexport_dlg.h"

//-------------------------   CHOOSE STARS DIALOG   --------------------------------

CChartDlg::CChartDlg(GtkWindow *pParent)
{
	GtkWidget *frame;

	// Dialog with buttons
	m_pDlg = gtk_dialog_new_with_buttons("Plot chart", pParent, 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR),
		GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, GTK_STOCK_SAVE, GTK_RESPONSE_NONE, NULL);

	// Dialog icon
	gchar *icon = get_icon_file("chart");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Chart
	frame = gtk_frame_new(NULL);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(m_pDlg)->vbox), frame, true, true, 0);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(g_object_ref(m_Chart.Handle())));
	gtk_widget_set_size_request(m_Chart.Handle(), 600, 400);
	m_Chart.ShowBadStars(false);
	m_Chart.EnableTracking(false);
	m_Chart.ShowImage(true);
	m_Chart.ShowChart(false);
	m_Chart.ShowNonMatchedStars(false);

	gtk_widget_show_all(GTK_DIALOG(m_pDlg)->vbox);

	// Signals:
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);
}

CChartDlg::~CChartDlg()
{
	gtk_widget_destroy(m_pDlg);
}

void CChartDlg::Execute()
{
	GtkWindow *pMainWnd = Global.pMainWnd->GetHandle();
	int refid;
	char *ppath, *tpath;
	const char *cat_file, *tmp_file;

	m_Chart.Clear();
	Global.pProject->GetReference(&refid, &cat_file);
	if (refid>0) {
		// Load reference frame
		tpath = Global.pProject->GetImageFile(refid);
		m_Chart.LoadImage(tpath);
		ppath = Global.pProject->GetPhotFile(refid);
		m_Chart.LoadChart(ppath, true);
		g_free(ppath);
		g_free(tpath);
	} else if (cat_file) {
		// Load catalogue file
		tmp_file = Global.pProject->GetTempCatFile();
		m_Chart.LoadChart(tmp_file, true);
		ppath = SetFileExtension(tmp_file, "fts");
		m_Chart.LoadImage(ppath);
		g_free(ppath);
	} else {
		ShowError(pMainWnd, "No reference file.");
		return;
	}

	// Set aperture
	m_Chart.SetAperture(Global.pProject->Aperture());

	// Set selection
	m_Chart.SetSelection(Global.pProject->Selection());

	gtk_dialog_run(GTK_DIALOG(m_pDlg));
}

void CChartDlg::response_dialog(GtkWidget *pDlg, gint response_id, CChartDlg *pMe)
{
	if (!pMe->OnResponseDialog(response_id)) 
		g_signal_stop_emission_by_name(pDlg, "response");
}

bool CChartDlg::OnResponseDialog(gint response_id)
{
	if (response_id == GTK_RESPONSE_NONE) {
		SaveChart();
		return false;
	}
	return true;
}

void CChartDlg::SaveChart()
{
	CChartExportDlg dlg(GTK_WINDOW(m_pDlg));
	dlg.Execute(&m_Chart);
}
