/**************************************************************

masterflat_dlg.cpp (C-Munipack project)
The 'Master flat-frame' dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <cmunipack.h>

#include "project.h"
#include "utils.h"
#include "configuration.h"
#include "masterflat_dlg.h"
#include "progress_dlg.h"
#include "main.h"
#include "proc_classes.h"

//-------------------------   MAIN WINDOW   --------------------------------

CMasterFlatDlg::CMasterFlatDlg(GtkWindow *pParent)
{
	GtkWidget *frame, *vbox;
	GSList *group;

	m_FilePath = NULL;

	// Dialog with buttons
	m_pDlg = gtk_dialog_new_with_buttons("Make master flat-frame", pParent, 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR),
		GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);

	// Dialog icon
	gchar *icon = get_icon_file("masterflat");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Frame
	frame = gtk_frame_new("Process");
	gtk_container_add (GTK_CONTAINER(GTK_DIALOG(m_pDlg)->vbox), frame);
	gtk_widget_set_size_request(frame, 300, -1);
	gtk_widget_show(frame);

	// Frame layout
	vbox = gtk_vbox_new(FALSE, 4);
	gtk_container_add(GTK_CONTAINER(frame), vbox);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 4);
	gtk_widget_show(vbox);

	// Options
	m_AllBtn = gtk_radio_button_new_with_label(NULL, "all files in current project");
	gtk_box_pack_start(GTK_BOX(vbox), m_AllBtn, TRUE, TRUE, 0);
	gtk_widget_show(m_AllBtn);
	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(m_AllBtn));

	m_SelBtn = gtk_radio_button_new_with_label(group, "selected files only");
	gtk_box_pack_start(GTK_BOX(vbox), m_SelBtn, TRUE, TRUE, 0);
	gtk_widget_show(m_SelBtn);

	// Default state
	GtkTreeSelection *pSel = Global.pMainWnd->GetSelection();
	if (gtk_tree_selection_count_selected_rows(pSel)>1) 
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_SelBtn), true);
	else
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_AllBtn), true);
}

CMasterFlatDlg::~CMasterFlatDlg()
{
	g_free(m_FilePath);
	gtk_widget_destroy(m_pDlg);
}

gboolean CMasterFlatDlg::foreach_all_files(GtkTreeModel *model, GtkTreePath *path, 
										GtkTreeIter *iter, gpointer userdata)
{
	GList **list = ((GList**)userdata);
	GtkTreeRowReference *rowref = gtk_tree_row_reference_new(model, path);
	*list = g_list_append(*list, rowref);
	return FALSE;
}

void CMasterFlatDlg::foreach_sel_files(GtkTreeModel *model, GtkTreePath *path, 
							   GtkTreeIter *iter, gpointer userdata)
{
	GList **list = ((GList**)userdata);
	GtkTreeRowReference *rowref = gtk_tree_row_reference_new(model, path);
	*list = g_list_append(*list, rowref);
}

void CMasterFlatDlg::Execute()
{
	int res;
	char msg[256];
	char *path, *file, *name;
	GtkWindow *pMainWnd = Global.pMainWnd->GetHandle();

	if (gtk_dialog_run(GTK_DIALOG(m_pDlg))!=GTK_RESPONSE_ACCEPT)
		return;
	gtk_widget_hide(m_pDlg);

	GtkWidget *pSaveDlg = gtk_file_chooser_dialog_new("Save master flat-frame to...",
		pMainWnd, GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(pSaveDlg), true);
	path = CConfig::GetStr("MasterFlat", "Folder", NULL);
	if (path) {
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(pSaveDlg), path);
		g_free(path);
	}
	name = CConfig::GetStr("MasterFlat", "Name", "masterflat");
	if (name) {
		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(pSaveDlg), name);
		g_free(name);
	}
	if (gtk_dialog_run(GTK_DIALOG(pSaveDlg))!=GTK_RESPONSE_ACCEPT) {
		gtk_widget_destroy(pSaveDlg);
		return;
	}
	file = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(pSaveDlg));
	path = g_path_get_dirname(file);
	CConfig::SetStr("MasterFlat", "Folder", path);
	g_free(path);
	name = g_path_get_basename(file);
	CConfig::SetStr("MasterFlat", "Name", name);
	m_FilePath = SetFileExtension(file, "fts");
	CConfig::SetStr("FlatCorr", "File", m_FilePath);
	g_free(file);
	gtk_widget_destroy(pSaveDlg);

	CProgressDlg::InitBuffer();

	m_FileList = NULL;
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_AllBtn))) {
		// All files
		GtkTreeModel *pList = Global.pProject->FileList();
		if (gtk_tree_model_iter_n_children(pList, NULL)>0) 
			gtk_tree_model_foreach(pList, foreach_all_files, &m_FileList);
		else
			ShowError(pMainWnd, "There are no files in the project.");
	} else {
		// Selected files
		GtkTreeSelection *pSel = Global.pMainWnd->GetSelection();
		if (gtk_tree_selection_count_selected_rows(pSel)>0) 
			gtk_tree_selection_selected_foreach(pSel, foreach_sel_files, &m_FileList);
		else
			ShowError(pMainWnd, "There are no selected files.");
	}
	if (m_FileList) {
		CProgressDlg pDlg(Global.pMainWnd->GetHandle(), "Reading files...");
		pDlg.SetMinMax(0, g_list_length(m_FileList));
		res = pDlg.Execute(ExecuteProc, this);
		if (res!=0) {
			char *msg = cmpack_formaterror(res);
			ShowError(pMainWnd, msg);
			cmpack_free(msg);
		} else if (!pDlg.Cancelled()) {
			sprintf(msg, "All %d file(s) was successfully processed.", m_InFiles);
			ShowInformation(pMainWnd, msg);
			Global.pMainWnd->ShowFile(m_FilePath);
		}
		g_list_foreach(m_FileList, (GFunc)gtk_tree_row_reference_free, NULL);
		g_list_free(m_FileList);
		m_FileList = NULL;
	}
	Global.pProject->Save();
	CConfig::Flush();
}

int CMasterFlatDlg::ExecuteProc(CProgressDlg *sender, void *userdata)
{
	return ((CMasterFlatDlg*)userdata)->ProcessFiles(sender);
}

int CMasterFlatDlg::ProcessFiles(CProgressDlg *sender)
{
	int frameid, res;
	char *fpath, *fbase, msg[128];
	GtkTreePath *path;
	CMasterFlatProc mflat;

	m_InFiles = 0;
	sender->Print("------ Make master flat frame ------");

	res = mflat.Open(sender, m_FilePath);
	if (res==0) {
		for (GList *node = m_FileList; node != NULL; node = node->next) {
			path = gtk_tree_row_reference_get_path((GtkTreeRowReference*)node->data);
			frameid = Global.pProject->GetFrameID(path);
			fpath = Global.pProject->GetImageFileName(path);
			sender->SetFileName(fpath);
			sender->SetProgress(m_InFiles++);
			sprintf(msg, "Frame #%d:", frameid);
			sender->Print(msg);
			res = mflat.Add(path);
			gtk_tree_path_free(path);
			g_free(fpath);
			if (res!=0 || sender->Cancelled()) 
				break;
		}
		if (res==0 && !sender->Cancelled()) {
			sender->SetTitle("Making output file...");
			fbase = g_filename_display_basename(m_FilePath);
			sender->SetFileName(fbase);
			g_free(fbase);
			res = mflat.Close();
		}
		if (sender->Cancelled())
			sprintf(msg, "Cancelled at the user's request");
		else
			sprintf(msg, "====== %d succeeded ======", m_InFiles);
		sender->Print(msg);
	} else {
		char *aux = cmpack_formaterror(res);
		sender->Print(msg);
		cmpack_free(aux);
	}
	return res;
}
