/**************************************************************

progress_dlg.h (C-Munipack project)
Progress dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef CMPACK_PROGRESS_DLG_H
#define CMPACK_PROGRESS_DLG_H

#include <gtk/gtk.h>
#include <glib/gthread.h>

typedef int ExecProc(class CProgressDlg *sender, void *user_data);

class CProgressDlg
{
public:
	CProgressDlg(GtkWindow *pParent, const char *Title);
	virtual ~CProgressDlg();

	static void InitBuffer(void);
	static void FreeBuffer(void);
	static GtkTextBuffer *GetBuffer(void) 
	{ return ms_Buffer; }
	static void OutputProc(const char *text, void *user_data);

	int Execute(ExecProc *pExecFn, void *user_data = NULL);

	void SetTitle(const char *title);
	void SetFileName(const char *filename);
	void SetMinMax(double min, double max);
	void SetProgress(double val);
	void Print(const char *text);

	bool Cancelled(void);

private:
	enum tEventCode
	{
		EVENT_MESSAGE,
		EVENT_TITLE,
		EVENT_FILENAME,
		EVENT_PROGRESS,
		EVENT_SUSPEND,
		EVENT_FINISHED
	};

	struct tMessage
	{
		tEventCode	event;
		gchar		*text;
		gint		int_val;
		gdouble		dbl_val;
	};

	GtkWidget	*m_pDlg, *m_Caption, *m_FileName, *m_TextView;
	GtkWidget	*m_Progress, *m_CancelBtn, *m_SuspendBtn;
	GThread		*m_pThread;
	GMutex		*m_pDataMutex;
	GCond		*m_pResume;
	GAsyncQueue	*m_pQueue;
	ExecProc	*m_pExecFn;
	gpointer	m_UserData;
	int			m_Result;
	double		m_Min, m_Max;
	GMainLoop	*m_Loop;
	bool		m_StopRq, m_Suspend, m_Waiting;

	static GtkTextBuffer *ms_Buffer;

	static void thread_proc(CProgressDlg *pMe);
	static void show_dialog(GtkWidget *widget, CProgressDlg *pDlg);
	static gboolean delete_event(GtkWidget *pWnd, GdkEvent *event, CProgressDlg *pDlg);
	static gboolean idle_func(CProgressDlg *pDlg);
	static void cancel_clicked(GtkButton *button, CProgressDlg *pDlg);
	static void suspend_clicked(GtkButton *button, CProgressDlg *pDlg);

	void InternalAddMessage(const char *text);
	void InternalSetTitle(const char *text);
	void InternalSetFileName(const char *text);
	void InternalSetProgress(double val);

	void PushMessage(tEventCode event, gint intval=0, gdouble dblval=0.0, gchar *text=NULL);
	tMessage *PopMessage(void);
	static void FreeMessage(tMessage *msg);
};

#endif
