/**************************************************************

utils.h (C-Munipack project)
Helper functions
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef MUNIWIN_UTILS_H
#define MUNIWIN_UTILS_H

#include <gtk/gtk.h>

/* PI number */
#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

// Get file types
enum tFileType
{
	FILE_TYPE_UNKNOWN	= 0,
	FILE_TYPE_IMAGE		= (1<<0),
	FILE_TYPE_PHOT		= (1<<1),
	FILE_TYPE_CAT		= (1<<2),
	FILE_TYPE_TABLE		= (1<<3),
	FILE_TYPE_READALL	= (1<<4),
	FILE_TYPE_ALL		= 0xFFFF
};

// Constants
#define MAX_APERTURES		12
#define MAX_SELECTION		64
#define MAX_OBJNAME_LEN		260
#define MAX_OBSNAME_LEN		260
#define MAX_LOCATION_LEN	260

// Color
typedef unsigned long tColor;

// Encode color
#define RGB(r, g, b) ((tColor)((r) | ((g)<<8) | ((b)<<16)))
#define RGBA(r, g, b, a) ((tColor)((r) | ((g)<<8) | ((b)<<16) | ((a)<<24)))

// Decode color components
#define GetR(color) ((unsigned char)((color)&0xFF))
#define GetG(color) ((unsigned char)(((color)>>8)&0xFF))
#define GetB(color) ((unsigned char)(((color)>>16)&0xFF))
#define GetA(color) ((unsigned char)(((color)>>24)&0xFF))

int Min(int a, int b);
int Max(int a, int b);
double Min(double a, double b);
double Max(double a, double b);

double LimitValue(double value, double min, double max);
int LimitValue(int value, int min, int max);

// Get path to the directory with user files
const char *get_user_dir(void);

// Get path to the directory with temporary files
const char *get_temp_dir(void);

// Get path to the directory with shared files
const char *get_share_dir(void);

// Get path to the icon file
char *get_icon_file(const gchar *icon);

// Get a button from dialog's action area by a response id
GtkWidget *get_dialog_widget_by_response(GtkWidget *pDlg, gint response_id);

// Make directory tree
bool force_directory(const gchar *path);

// Show user's manual
void ShowHelp(GtkWindow *pParent);

// Show message box with error message
void ShowError(GtkWindow *pParent, const char *message);

// Show message box with warning message
void ShowWarning(GtkWindow *pParent, const char *message);

// Show message box with information
void ShowInformation(GtkWindow *pParent, const char *message);

// Show configuration message box
bool ShowConfirmation(GtkWindow *pParent, const char *message);

// Set/change file extension
char *SetFileExtension(const char *path, const char *ext);

// Check file extension
bool CheckFileExtension(const char *path, const char *ext);

// Autodetect file type
tFileType FileType(const char *path);

// Make a file name from object's name (leaves out all characters
// that are not allowed in the file name. Returns copy of a given 
// string, use g_free() function to free it.
char *MakeFileName(const char *objname, const char *ext);

// Return nearest integer number with limit checking
int RoundToInt(double x);

// Get aperture radius
double GetApertureRadius(int aperture);

// Compare strings, they can be NULL
int StrCmp0(const char *str1, const char *str2);

#endif
