/**************************************************************

varfind_dlg.h (C-Munipack project)
The 'Find variables' dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef CMPACK_VARFIND_DLG_H
#define CMPACK_VARFIND_DLG_H

#include <gtk/gtk.h>
#include <cmunipack.h>

#include "graph.h"
#include "graph_data.h"
#include "chart.h"

class CVarFindDlg
{
public:
	CVarFindDlg(GtkWindow *pParent);
	~CVarFindDlg();

	void Execute();

private:
	// Display modes
	enum tDispMode
	{
		DISP_IMAGE,
		DISP_CHART,
		DISP_FULL
	};

	// Selection mode
	enum tSelectMode
	{
		SELECT_VAR,
		SELECT_COMP
	};
	
	GtkWidget		*m_pDlg, *m_Toolbar, *m_Status, *m_Export;
	CGraph			m_MagDevGraph, m_LightCurveGraph;
	CGraphData		m_MagDevData, m_LightCurveData;
	CmpackAllFile	*m_AllFile;
	CmpackTable		*m_MagDevTable, *m_LightCurveTable;
	CChart			m_Chart;
	int				m_VarID, m_CompID, m_LastStarIndex, m_LastPtIndex;
	GSList			*m_FileList;
	int				m_Aperture;
	GtkWidget		*m_VarBtn, *m_CompBtn, *m_SaveMagDev, *m_SaveCurve, *m_SaveChart;
	tSelectMode		m_SelectMode;
	double			m_MinMag, m_MaxMag, m_MaxDev, m_JDMin, m_JDMax;
	bool			m_Updating, m_Cancelled;
	GtkWidget		*m_ShowImage, *m_ShowChart, *m_ShowFull;
	tDispMode		m_DispMode;

	static void response_dialog(GtkDialog *pDlg, gint response_id, CVarFindDlg *pMe);
	static int ExecuteProc(class CProgressDlg *sender, void *user_data);
	static gboolean foreach_all_files(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, GSList **list);
	static void toggled(GtkToggleButton *pBtn, CVarFindDlg *pDlg);
	static void clicked(GtkButton *pBtn, CVarFindDlg *pDlg);
	static void ChartCallback(int message, int wparam, void* lparam, void* cb_data);
	static void GraphCallback(int message, int wparam, void* lparam, void* cb_data);

	void SetDisplayMode(tDispMode mode);
	void RebuildData(GtkWindow *pParent);
	int ProcessFiles(class CProgressDlg *sender);
	bool OnResponseDialog(gint response_id);
	void OnToggled(GtkToggleButton *pBtn);
	void OnClicked(GtkButton *pBtn);
	void SetMode(tSelectMode mode);
	void SaveMagDev(void);
	void SaveChart(void);
	void SaveCurve(void);
	void Export(void);
	void OnChartMouseMove(void);
	void OnChartLButtonClick(const GdkEventButton *event);
	void OnGraphMouseMove(void);
	void OnGraphLButtonClick(const GdkEventButton *event);
	void UpdateSelection(void);
	void UpdateLightCurve(void);
};

#endif
